/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: matroid_from_graph.cc 9302 2009-09-04 06:56:21Z sherrmann $
*/

#include <list>

#include <Graph.h>
#include <polymake.h>
#include <Matrix.h>
#include <Rational.h>
#include <PowerSet.h>
#include <linalg.h>

namespace polymake { namespace matroid {


perl::Object matroid_from_graph(perl::Object g)
{
  const Graph<> graph=g.give("ADJACENCY");
  const int n_elements=g.give("N_EDGES");
  int r=g.give("N_NODES");
  --r;
  std::list< Set<int> > bases;
  int n_bases=0;



  Array<int> start_nodes(n_elements); //the starting nodes of the edges
  Array<int> end_nodes(n_elements); //the ending nodes of the edges
  
  int l=0;
  for (Entire< Edges<Graph<> > >::const_iterator i=entire(edges(graph)); !i.at_end(); ++i) {
    start_nodes[l]=i.from_node();
    end_nodes[l++]=i.to_node();
  }
  //test for all subsets of size r if they are spanning
  for (Entire< Subsets_of_k <sequence> >::const_iterator i=entire(all_subsets_of_k(sequence(0,n_elements),r)); !i.at_end(); ++i) {
    Array<bool> a(r+1);
    bool is_spanning=true;
    const Set<int> basis=*i;
    for (Entire<Set<int> >::const_iterator j=entire(basis); !j.at_end(); ++j)
      a[start_nodes[*j]]=a[end_nodes[*j]]=true;

    for (int k=0;is_spanning&&k<n_elements;++k)
      is_spanning|=a[k];
    if (is_spanning) {
      bases.push_back(basis);
      ++n_bases;
    }
  }

  perl::Object m("Matroid");  
  m.take("BASES") << bases;
  m.take("N_BASES") << n_bases;
  m.take("RANK") << r;
  m.take("N_ELEMENTS") << n_elements;
  return m;
}
  
  UserFunction4perl("# Category: Producing from scratch\n"
		    "# Creates a graphical matroid from a @a graph.\n"
		    "# args: graph \n",
		    &matroid_from_graph, "matroid_from_graph(graph::Graph)");

} }

