/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: uniform_matroid.cc 9302 2009-09-04 06:56:21Z sherrmann $
*/

#include <list>

#include <polymake.h>
#include <Matrix.h>
#include <Rational.h>
#include <PowerSet.h>
#include <Array.h>

namespace polymake { namespace matroid {


perl::Object uniform_matroid(const int r, const int n)
{

   if (n < 1)
      throw std::runtime_error("uniform_matroid: at least 1 element requiered");
   if (r < 1 || r > n)
      throw std::runtime_error("uniform_matroid: 1 <= r <= n required");


  perl::Object m("Matroid");

  m.take("N_ELEMENTS")<< n;
  m.take("RANK")<<r;
  // we already know the number of bases
  const int n_bases=Integer::binom(n,r);
  m.take("N_BASES") << n_bases;

  Array<Set<int> > bases(n_bases);
  int l=0;
  for (Entire< Subsets_of_k <sequence> >::const_iterator i=entire(all_subsets_of_k(sequence(0,n),r)); !i.at_end(); ++i)
    bases[l++]=*i;


  m.take("BASES") << bases;
  return m; 
}

UserFunction4perl("# Category: Producing from scratch\n"
		  "# Creates the uniform matroid of rank @a r with @a n elements\n"
		  "# args: r, n \n",
		  &uniform_matroid, "uniform_matroid");

} }

