/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: cayley_polytope.cc 9319 2009-09-16 09:17:24Z sherrmann $
*/


#include <polymake.h>
#include <Integer.h>
#include <Rational.h>
#include <Matrix.h>
#include <ListMatrix.h>
#include <Vector.h>

namespace polymake { namespace polytope {

    perl::Object cayley_polytope ( const Array<perl::Object> & pp, perl::OptionSet options ) {

      int d = pp.size();

      Entire< Array<perl::Object> >::const_iterator pp_it = entire(pp);

      if ( !pp_it->give("LATTICE") )
	throw std::runtime_error("cayley polytope construction only defined for lattice polytopes");

      perl::Object p_out(pp_it->type());
      std::string names = pp_it->name();
      ListMatrix< Vector<Integer> > U = pp_it->give("VERTICES | POINTS");
      U |= repeat_row(unit_vector<Integer>(d,0),U.rows());

      int i = 1;
      while ( !(++pp_it).at_end() ) {
	if ( !pp_it->give("LATTICE") )
	  throw std::runtime_error("cayley polytope construction only defined for lattice polytopes");

	Matrix<Integer> V = pp_it->give("VERTICES | POINTS");
	U /= V|repeat_row(unit_vector<Integer>(d,i++),V.rows());

	names += ", ";
	names += pp_it->name();
      }

      p_out.set_description() << "cayley polytope of the polytopes " << names << endl;
      if ( options["proj"] ) 
	p_out.take("POINTS") << U.minor(All,~range(U.cols()-1,U.cols()-1));
      else
	p_out.take("POINTS") << U;

      return p_out;
    }

    UserFunctionTemplate4perl("# Category: Producing a new polyhedron from others \n"
		      "# Construct the cayley polytope of a set of lattice polytopes p_1, ..., p_k"
		      "# which is the convex hull of p_1\times e_1, ..., p_k\times e_k"
                      "# where e_1, ..., e_k are the standard unit vectors in R^k."
                      "# In this representation the last k coordinates always add up to 1"
		      "# the option @a proj projects onto the complement of the last coordinate.",
		      "cayley_polytope(Polytope<Rational> +; {proj => 0} )");

  } }

