/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: gkz_vector.cc 9298 2009-09-02 23:10:56Z gawrilow $
*/

#include <Matrix.h>
#include <Rational.h>
#include <polymake.h>
#include <Array.h>
#include <Set.h>
#include <linalg.h>

namespace polymake { namespace polytope {

Vector<Rational> gkz_vector(const Matrix<Rational>& vert, const Array< Set<int> >& triang)
{
   Vector<Rational> gkz(vert.rows(),0);

   // go through all simplices
   for (Entire< Array< Set<int> > >::const_iterator i=entire(triang); !i.at_end(); ++i) {
      const Rational v=abs(det(vert.minor(*i,All)));
      for (Entire< Set<int> >::const_iterator j=entire(*i); !j.at_end(); ++j)
	 gkz[(*j)]+=v;
   }

   return gkz;
}

   UserFunction4perl("# Compute the GKZ-Vector of a @a triangulation  of @a points \n"
		     "# see Chapter 7 in Gelfand, Kapranov, and Zelevinsky: Discriminants, Resultants and\n"
		     "# Multidimensional Determinants, Birkhäuser 1994\n"
		     "# args: triangulation, points"		
		     "# @author: Sven Herrmann",
		     &gkz_vector,"gkz_vector");
} }
