/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: triang_boundary.cc 9298 2009-09-02 23:10:56Z gawrilow $
*/

#include <polymake.h>
#include <Array.h>
#include <PowerSet.h>
#include <IncidenceMatrix.h>

namespace polymake { namespace polytope {

   typedef Array< PowerSet<int> > triang_boundary_type;

   triang_boundary_type triang_boundary(const Array< Set<int> >& triang, const IncidenceMatrix<>& vif)
   {
     IncidenceMatrix<> VIF=vif;
     const int dim=triang[0].size()-1;		// #vertices(simplex) == dim+1
     triang_boundary_type triang_boundary(VIF.rows());
     
     // simplicial facets need not be triangulated
     for (Entire< Rows< IncidenceMatrix<> > >::iterator f=entire(rows(VIF)); !f.at_end(); ++f)
       if (f->size() == dim) {
	 triang_boundary[f.index()].push_back(*f);
	 f->clear();
       }
     
     for (Entire< Array< Set<int> > >::const_iterator t=entire(triang); !t.at_end(); ++t) {
       typedef Subsets_less_1<const Set<int>&> faces;
       for (Entire<faces>::const_iterator face=entire(all_subsets_less_1(*t)); !face.at_end(); ++face) {
	 faces::value_type::const_iterator v=entire(*face);
	 Set<int> common_facets=VIF.col(*v);
	 for (++v; !v.at_end() && !common_facets.empty(); ++v)
	   common_facets *= VIF.col(*v);
	 if (!common_facets.empty())
	   triang_boundary[common_facets.front()] += *face;
       }
     }
     
     return triang_boundary;
   }

   UserFunction4perl("# For a given @a triangulation, find the trace on its surface (triangulation of facets).\n"
		     "# args: points_in_facets triangulation",
		     &triang_boundary,"triang_boundary");
} }
