/* Copyright (c) 1997-2009
   Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
   http://www.polymake.de

   This program is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2, or (at your option) any
   later version: http://www.gnu.org/licenses/gpl.txt.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
--------------------------------------------------------------------------------
   $Project: polymake $$Id: triang_sign.cc 9298 2009-09-02 23:10:56Z gawrilow $
*/

#include <polymake.h>
#include <Matrix.h>
#include <Vector.h>
#include <Array.h>
#include <PowerSet.h>
#include <linalg.h>

namespace polymake { namespace polytope {

   template <typename Scalar>
   Array<int> triang_sign(const Array< Set<int> >& Triangulation, const Matrix<Scalar>& Points)
   {
      Array<int> signs(Triangulation.size());
      Array<int>::iterator s=signs.begin();
      for (typename Entire< Array< Set<int> > >::const_iterator t=entire(Triangulation);  !t.at_end();  ++t, ++s)
	 *s=sign(det(Points.minor(*t,All)));
      return signs;
   }

   // Point C is assumed to be an interior point of a polytope.
   // Since we are interested in simplices visible "from the outside", we invert the signs here.

   template <typename Scalar>
   Array< Array<int> > triang_sign(const Array< PowerSet<int> >& TriangBoundary, const Matrix<Scalar>& Points, const Vector<Scalar>& C)
   {
      Array< Array<int> > signs(TriangBoundary.size());
      Array< Array<int> >::iterator sf=signs.begin();
      for (Entire< Array< PowerSet<int> > >::const_iterator facet_t=entire(TriangBoundary);  !facet_t.at_end();  ++facet_t, ++sf) {
	 sf->resize(facet_t->size());
	 Array<int>::iterator s=sf->begin();
	 for (typename Entire< PowerSet<int> >::const_iterator t=entire(*facet_t);  !t.at_end();  ++t, ++s)
	    *s=-sign(det(Points.minor(*t,All) / C));
      }
      return signs;
   }

   FunctionTemplate4perl("triang_sign(Array, Matrix)");
   FunctionTemplate4perl("triang_sign(Array, Matrix, Vector)");
} }
